from ._integration import OdooIntegration


class PartnerModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_partner_by_id(self, partner_id, fields=None):
        response = self.read("res.partner", [partner_id], fields)
        return response

    def get_partner_id_by_name(self, partner_name, fields=None):
        response = self.search("res.partner", [[["name", "=", partner_name]]], fields)
        if response:
            return response[0]
        raise Exception(f"Partner '{partner_name}' not found")

    def update_partner(
        self,
        partner_id: int,
        partner_data: dict,
    ):
        response = self.update("res.partner", partner_id, partner_data)
        return response

    def create_partner_from_scratch(
        self,
        partner_data: dict,
    ):
        response = self.create("res.partner", [partner_data])
        return response

    def create_partner(
        self,
        name,
        street,
        city,
        state_id,
        country_id,
        zip_code,
        nif,
        contact_type,
        anf_code,
        pharmacy_name,
        owner_name,
        phone,
        mobile,
        email,
        list_category_id,
        list_crm_tag_id,
        vendor_user_id,
        reference,
        language="pt_PT",
        is_company=True,
        company_type="company",
    ):

        partner_data = {
            "name": name,
            "street": street,
            "city": city,
            "state_id": state_id,  # "Lisboa", # aqui deve-se buscar o state_id pelo nome do estado
            "country_id": country_id,  # "Portugal", # aqui deve-se buscar o country_id pelo nome do país
            "zip": zip_code,
            "vat": nif,  # NIF
            "x_studio_tipo_de_contacto_1": contact_type,  # Tipo de Contacto
            "x_studio_cdigo_anf_1": anf_code,  # apenas para tipo farmácia
            "x_studio_nome_da_farmcia_1": pharmacy_name,  # apenas para tipo farmácia
            "x_studio_nome_do_proprietrio_1": owner_name,  # apenas para tipo farmácia
            "phone": phone,
            "mobile": mobile,
            "email": email,
            "lang": language,  # idioma
            "category_id": list_category_id,  # Etiquetas / labels - models: res.partner.category - fields: name, active (booleean), parent_id (categoria pai), display_name
            "x_studio_etiquetas_crm": list_crm_tag_id,  # model: crm.tag - fields: name, id, color
            "user_id": vendor_user_id,  # models: res.users - fields: name
            "ref": reference,  # Referência
            "is_company": is_company,  # se é empresa ou não
            "company_type": company_type,  # tipo de empresa
        }
        response = self.create("res.partner", [partner_data])
        return response

    def get_and_read_all_partners(self, fields=None, limit=10, offset=0):
        response = self.search_read("res.partner", [], fields or [], limit, offset)
        return response

    def count_partners_by_any_filter(self, filter: list = None):
        response = self.search_count("res.partner", [filter or []])
        return response

    def get_and_read_partners_by_any_filter(
        self, filter: list, fields=None, limit=10, offset=0
    ):
        response = self.search_read(
            "res.partner", [filter], fields or [], limit, offset
        )
        return response
    
    def action_archive_partner(self, partner_id):
        response = self.execute_action("res.partner", "action_archive", partner_id)
        return response



from ._account_tax import AccountTaxModel
from ._country import CountryModel
from ._crm_tag import CRMTagModel
from ._integration import OdooIntegration
from ._partner_category import PartnerCategoryModel
from ._partner import PartnerModel
from ._product_category import ProductCategoryModel
from ._product_template import ProductTemplateModel
from ._product import ProductModel
from ._stock import StockModel
from ._sale_order import SaleOrderModel
from ._user import UserModel
from ._analytic_account import AnalyticAccountModel
from ._payment_term import PaymentTermModel
from ._company import CompanyModel
from ._purchase_order import PurchaseOrderModel
from ._account_move import AccountMoveModel


from ._integration import OdooIntegration


class SaleOrderModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_sale_order_by_any_filter(self, filter: list):
        response = self.search("sale.order", [filter])
        return response

    def get_sale_order_by_state(self, state: list):
        response = self.search("sale.order", [[["state", "in", state]]])
        return response

    def get_sale_order_by_id(self, sale_order_id, fields=None):
        response = self.read("sale.order", [sale_order_id], fields)
        return response

    def get_sale_order_line_list_by_sale_order_id(self, sale_order_id):
        response = self.search("sale.order.line", [[["order_id", "=", sale_order_id]]])
        return response

    def get_sale_order_line_by_id(self, sale_order_line_id):
        response = self.read("sale.order.line", [sale_order_line_id])
        return response

    def create_sale_order_line(self, sale_order_line_data: dict):
        # sale_order_line = {
        #     "order_id": sale_order_id,
        #     "product_id": product_id,
        #     "product_uom_qty": product_quantity,
        #     "price_unit": price_unit,
        #     "discount": discount,
        #     # "x_studio_many2one_field_ZK6OA": lote_id,  # id do lote
        #     # "x_studio_data_de_validade": "2024-02-21", # data de validade (se não for passado, pega a data do lote)
        # }
        # if lote_id:
        #     sale_order_line["x_studio_many2one_field_ZK6OA"] = lote_id
        response = self.create("sale.order.line", [sale_order_line_data])
        return response

    def create_sale_order(self, sale_order_data: dict):
        # sale_order = {
        #     "partner_id": partner_id,
        #     "company_id": company_id,  # serão duas empresas, addo pharm e addo pharm distribuição
        #     "warehouse_id": warehouse_id,  # warehouse de distribuiçãos (armazem)
        #     "analytic_account_id": analytic_account_id,
        # }
        response = self.create("sale.order", [sale_order_data])
        return response

    def confirm_sale_order(self, sale_order_id):
        response = self.execute_action("sale.order", "action_confirm", sale_order_id)
        return response

    def certify_sale_order(self, sale_order_id):
        response = self.execute_action("sale.order", "certify", sale_order_id)
        return response

    def create_invoice_from_sale_order(self, sale_order_id):
        # sale_order_id is used as active_id (context)
        extra_context = {"active_id": sale_order_id, "active_ids": [sale_order_id], "active_model": "sale.order"}
        response = self.execute_action("sale.advance.payment.inv", "create", [], extra_context)
        return response
    

    def confirm_invoice_from_sale_order(self, invoice_id, sale_order_id):
        # sale_order_id is used as active_id (context)
        extra_context = {"active_id": sale_order_id, "active_ids": [sale_order_id], "active_model": "sale.order"}
        response = self.execute_action("sale.advance.payment.inv", "create_invoices", invoice_id, extra_context)
        return response
    
    def certify_invoice_from_sale_order(self, sale_order_id):
        response = self.execute_action("sale.order", "certify", sale_order_id)
        return response
    
    def get_sale_order_message(self, message_id):
        response = self.read("mail.message", [message_id])
        return response

    def get_sale_order_attachment(self, attachment_id):
        response = self.read("ir.attachment", [attachment_id])
        return response[0]

    def confirm_invoice_by_button_action_by_invoice_id(self, invoice_id):
        response = self.execute_action("account.move", "action_post", invoice_id)
        return response
    
    def get_invoice_id_from_sale_order(self, sale_order_id):
        response = self.execute_action("sale.order", "action_view_invoice", sale_order_id)
        return response["res_id"]

    def invoice_force_creation(self, invoice_id):
        response = self.update("account.move", invoice_id, {"force_open": True})
        return response
    
    def invoice_insert_footer_notes(self, invoice_id, notes):
        response = self.update("account.move", invoice_id, {"footer_notes": notes})
        return response
    
    # def invoice_insert_narration(self, invoice_id, narration):
    #     response = self.update("account.move", invoice_id, {"narration": narration})
    #     return response
    
    # def get_invoice(self, invoice_id):
    #     response = self.read("account.move", [invoice_id])
    #     return response[0]


from ._integration import OdooIntegration


class AccountTaxModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_tax_id_by_name(self, tax_name, company_id=None):
        if company_id:
            response = self.search("account.tax", [[["name", "=", tax_name], ["company_id", "=", company_id]]])
        else:    
            response = self.search("account.tax", [[["name", "=", tax_name]]])
        if response:
            return response[0]
        raise Exception(f"Tax '{tax_name}' not found")

    def get_tax_by_id(self, tax_id, fields=None):
        response = self.read("account.tax", [tax_id], fields)
        return response
    



from ._integration import OdooIntegration


class AnalyticAccountModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_analytic_account_id_by_name(self, analytic_account_name):
        response = self.search("account.analytic.account", [[["name", "=", analytic_account_name]]])
        if response:
            return response[0]
        raise Exception(f"Analytic account name '{analytic_account_name}' not found")



from ._integration import OdooIntegration


class AccountMoveModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def confirm_invoice_purchase_order(self, invoice_id):
        response = self.execute_action("account.move", "action_post", invoice_id)

    def invoice_insert_footer_notes(self, invoice_id, notes):
        response = self.update("account.move", invoice_id, {"footer_notes": notes})
        return response

    def invoice_insert_product_line(self, invoice_id, data):
        response = self.update("account.move", invoice_id, data)
        return response

    def get_invoice_by_id(self, invoice_id):
        response = self.read("account.move", [invoice_id])
        return response

    def get_invoice_line_by_id(self, account_move_line_id):
        response = self.read("account.move.line", [account_move_line_id])
        return response
    
    def update_invoice_line(self, invoice_id, data):
        response = self.update("account.move", invoice_id, data)
        return response


from ._integration import OdooIntegration


class CountryModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def get_country_id_by_code(self, country_code):
        response = self.search("res.country", [[["code", "=", country_code]]])
        if response:
            return response[0]
        raise Exception(f"Country '{country_code}' not found")
    
    def get_country_id_by_name(self, country_name):
        response = self.search("res.country", [[["name", "=", country_name]]])
        if response:
            return response[0]
        raise Exception(f"Country '{country_name}' not found")

    def get_country_state_by_id(self, state_id):
        response = self.read("res.country.state", [state_id])
        if response:
            return response[0]
        raise Exception(f"State '{state_id}' not found")

    def get_country_state_id_by_name(self, state_name):
        response = self.search("res.country.state", [[["name", "=", state_name]]])
        if response:
            return response[0]
        raise Exception(f"State '{state_name}' not found")



from ._integration import OdooIntegration


class ProductTemplateModel(OdooIntegration):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _validate_detailed_type(self, detailed_type):
        """
        product = Artigo Armazenável
        consul = Consumível
        service = Serviço
        """
        if detailed_type not in ["product", "consul", "service"]:
            raise Exception("Detailed type not valid")

    def _validate_invoice_policy(self, invoice_policy):
        """
        orders = Quantidades Pedidas
        delivery = Quantidades Entregues
        """
        if invoice_policy not in ["order", "delivery"]:
            raise Exception("Invoice policy not valid")

    def _validate_expense_policy(self, expense_policy):
        """
        no = Não
        cost = A Custo
        sales_price = Preço de vendas
        """
        if expense_policy not in ["no", "cost", "sales_price"]:
            raise Exception("Expense policy not valid")

    def _validate_tracking(self, tracking):
        """
        none = Sem rastreio
        lot = Por Lotes
        serial = Por número de série
        """
        if tracking not in ["lot", "serial", None]:
            raise Exception("Tracking not valid")

    def _validate_purchase_method(self, purchase_method):
        """
        receive = Nas quantidades recebidas
        purchase = Nas quantidades pedidas
        """
        if purchase_method not in ["purchase", "receive"]:
            raise Exception("Purchase method not valid")

    def get_product_by_id(self, product_id):
        response = self.read("product.template", [product_id])
        return response

    def get_product_id_by_reference(self, reference_id):
        response = self.search(
            "product.template", [[["default_code", "=", reference_id]]]
        )
        if response:
            return response[0]
        raise Exception(f"Product '{reference_id}' not found")

    def create_product(
        self,
        name,
        default_code,
        detailed_type,
        invoice_policy,
        expense_policy,
        list_price,
        taxes_id,
        standard_price,
        categ_id,
        tracking,
        purchase_method,
        supplier_taxes_id,
        use_expiration_date=True,
        sale_ok=True,
        purchase_ok=True,
        barcode=None,
    ):
        self._validate_detailed_type(detailed_type)
        self._validate_invoice_policy(invoice_policy)
        self._validate_expense_policy(expense_policy)
        self._validate_tracking(tracking)
        self._validate_purchase_method(purchase_method)

        product_data = {
            "name": name,
            "default_code": default_code,
            "detailed_type": detailed_type,
            "invoice_policy": invoice_policy,
            "expense_policy": expense_policy,
            "list_price": list_price,
            "taxes_id": taxes_id,
            "standard_price": standard_price,
            "categ_id": categ_id,
            "tracking": tracking,
            "purchase_method": purchase_method,
            "supplier_taxes_id": supplier_taxes_id,
            "use_expiration_date": use_expiration_date,
            "sale_ok": sale_ok,
            "purchase_ok": purchase_ok,
            "barcode": barcode if barcode else False,
        }
        response = self.create("product.template", [product_data])
        return response
